/*
 * Created on 11.12.2004
 *
 */
package sibiriaGame;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Random;
import java.util.Set;



/**
 *  This class is part of the "Travelling in Sibiria" application. 
 *  "Travelling in Sibiria" is a adventure game with a userinterface.
 * 
 *  This class creates all rooms, the items contained in each room, 
 *  the persons from each room and initializes the player, creates the parser 
 *  calls the userinterface and starts the game.  
 *  It also evaluates and executes the commands that the parser returns.
 * 
 * @author  Vera Demberg
 * @version 1.0 (December 2004)
 */
public class GameEngine
{	
	private Random randomNumber = new Random();
    private Parser parser;
    private Room currentRoom;
    private UserInterface gui;
    private String currency = "Roubles";
	private Player player;
	private HashSet rooms= new HashSet();
	private int photos =0;
	private int maxphotos=7;
	private int maxfreeze=10;
	private int maxthirst=10;
	private boolean spike_shoes_given= false;//whether the player got spike shoes from his room mate
	private boolean game_over=false;
	//******************ROOMS******************************
	private Room airport, change_office_open, change_office_closed,hotel_room;
	private OpenAir citycenter,baikal,bus_station,middle_of_nowhere,lenin_monument_with_tourists, on_baikal_lake,city_center_shops,village,ice_cave;
	private Transport hotel,lenin_monument,bus, busback,middle_of_nowhere_bus,outside1, outside2;
	private Shop banja, city_shop, fish_shop,airport_shop,newspaper_shop;
	//*****************ITEMS*******************************
	private Item busticket, busticket2, busticket3,newcamera,ticket, taxiticket,taxiticket2,keys, camera, towel, soap;
	private ConsumableItem fish,food;
	private DrinkableItem  bottle_of_vodka,bottle_of_vodka2;
	private FreezableItem  bottle_of_water,bottle_of_water2;
	private Money money;
    private WearableItem  scarf, shapka, gloves;
    private Suitcase suitcase;
    private Dictionary dictionary,dictionary2;
    private Spikeshoes spikeshoes,spikeshoes_mate;
	//****************CHARACTERS************************
	private Seller newspaper_woman,airport_shop_seller,city_shop_seller,fish_shop_seller;
	private Person change_woman,banja_woman,hotel_waiter,soldier,tourist,orthodox_woman;
	private Chatter person_waiting_for_bus, person_waiting_for_bus2;
	private Driver busdriver1, busdriver2, busdriver3, taxidriver, taxidriver2;
	
	/**
	 * main method
	 * @param args
	 */
	public static void main(String[] args) {
		GameEngine game = new GameEngine();		

	}
	

    /**
     * Constructor for objects of class GameEngine
     */
	
	public GameEngine()
    {
		
        parser = new Parser();        
        createItems();
        createCharacters();
        createRooms();    
        equipPlayer();//gives to the player his initial items, the start amount of money and the initial time of day.
        ((Money) money).setMoney(player.getMoney()); //this can't be done in either createItems nor createCharacters because money is a attribute of the player and an item at a time.
        Iterator iter = rooms.iterator();//iterates over all created rooms to add some last information which was not available at the time of creation
        while (iter.hasNext()){        	
        	Room s=	(Room) iter.next();
        	s.initializePerson(); //assigns to each person the items sold where appropriate
        	s.setPlayer(player);//makes the public player methods accessible to rooms 
        }
        gui= new UserInterface(this);//creates UserInterface and makes GameEngine accessible from userInterface.
        printWelcome();//prints out a welcome and the description of the first room to the output text area of the user Interface.
        String rules = "Welcome to the adventure \"Travelling in Sibiria\". " +
        		"\nYou are a tourist who has just luckily arrived with this machine. Your goal in this adventure" +
        		"\nis to get around and take photos. During your expedition you will have to solve some problems " +
        		"\nlike not dying of cold, getting enough to eat and to drink and managing well the money you have." +
				"\nYou should pay attention to the following particularities:" +
				"\nYou will lose the game if you exceed the maximal freeze level, exceed the maximal thirst level or" +
				"\n have no money left. If you are hungry, you will freeze faster. " +
				"\nGet to your hotel before 7pm! Otherwise you risk to be robbed." +
				"\nIf you get to your hotel after 5pm, the time will change to 9am next morning and your freezing level," +
				"\nhunger level and thirst level will be reseted."+
				"\n\nSome useful hints: try to negotiate before you buy things, talk to the people you meet, ..." +
				"\nto perform your actions you can use the buttons provided or type your commands in the input line." +
				"\nIf you need help you can use the help button or click on the buttons on the right of the window," +
				"\nwhich represent the items you can posses  or can get in the actual room." +
				"\nFor the rest: find out yourself!";
        gui.startScreen("flugzeug.JPG",rules);//gives information about with image and which text to display as a first screen.
    }
    

   
	/**
	 * This method initializes all characters of the game with a name and a description
	 *  and assigns to them their special sentences. 
	 * 
	 */
	private void createCharacters() {
		airport_shop_seller= new Seller("Ljudmilla", "She understands English a bit from her work in the airport shop.");			
		
		taxidriver= new Driver("Wladimir","He is the driver of this first cab. Not too friendly.");	//first Argument: name, second arguemtn: description of character
		taxidriver.setSentence("init","I'll drive you in half an hour.");//first Argument: type of sentence, second Argument sentenceTemplate, could contain holes of type ||NOUN||, for more information see TemplateLanguageGenerator class
		taxidriver.setKeyWordAndAnswer("hotel","No, just to the city center.");//first Argument: keyword, second Arguemnt: sentence which is the reaction on that keyword
		
		taxidriver2 = new Driver("Pjotr","He is the driver this cab. He hopes to get some clients today.");
		taxidriver2.setKeyWordAndAnswer("hotel","Yes, of course. Irkutsk has one hotel, I'll bring you there. It's right in the center of Irkutsk.");
		
		change_woman = new Person("Anja","She prefers to chat with her friend instead of changing money.");
		change_woman.setSentence("question","Want to change some money?");
		change_woman.setKeyWordAndAnswer("Pounds","But I can't change Roubles to Pounds, just Pounds to Roubles or Roubles to Euros.");
		change_woman.setKeyWordAndAnswer("Roubles","But I can't change Roubles to Pounds, just Pounds to Roubles or Roubles to Euros.");
		
		banja_woman = new Person("Aleksandra","She sells the tickets for the banja and can be asked if there are problems.");
		banja_woman.setSentence("reply","ah, ty tourist...");
		banja_woman.setSentence("question","polotentse jest?");
		banja_woman.setSentence("question","i milo, gde?");
		banja_woman.setSentence("reply","ojojojojojoj");
		
		soldier = new Person("Lev","Checks tourists who want to get to the Lenin monument. Could need more money");
		soldier.setSentence("init","It's not allowed to take photos of the Lenin monument.");
		soldier.setKeyWordAndAnswer("photo","It's not allowed to take photos of the Lenin monument.");
		soldier.setSentence("init","There are a lot of people around here.");
		soldier.setKeyWordAndAnswer("possibility","There are a lot of people around here...");
		soldier.setKeyWordAndAnswer("do","There are a lot of people around here...");
		soldier.setKeyWordAndAnswer("please","I could think about it...there are a lot of people around here...");
		soldier.setKeyWordAndAnswer("much","rhmmm... 200 Rourhmmbles...rhmmm");
		soldier.setKeyWordAndAnswer("money","After all... yes");
		
		orthodox_woman = new Person("Marija","She is a very pious orthodox woman.");//gives you a photo as a present
		orthodox_woman.setSentence("init","Vot, fotografij! beritje, beritje! choroshoj irkutskij chram!");
		
		hotel_waiter = new Person("Maksim","He works in the reception.");
		hotel_waiter.setKeyWordAndAnswer("room","The room is 300 Roubles per night. You can take the keys here.");
		hotel_waiter.setKeyWordAndAnswer("where","It's the first room you come to.");
		hotel_waiter.setSentence("init","A shared room is 300 Roubles per night. There aren't any other rooms available at the moment.");
		
		tourist = new Person("John","He has already arrived some days earlier and might give you some advises what to see.");	
		tourist.setSentence("init","I strongly recommend you to go to the Baikal lake!\nIt's so beautiful! But if you do it, you should take spike shoes with you, the ice is so slippery.\nI'll let you my spike shoes. Here you are.");
		
		newspaper_woman = new Seller("Mira","She sells newspapers and finds that quite boring.");
		
		city_shop_seller = new Seller("Yelena","She sells all kinds of products.");
		
		fish_shop_seller = new Seller("Viktoria","She sells fresh hot baikal omul and some vodka.");
		
		person_waiting_for_bus = new Chatter("Igor","He also waits for bus.");
		person_waiting_for_bus.setKeyWordAndAnswer("you","I'm going to the Baikal lake with my friend for fishing.");
		person_waiting_for_bus.setKeyWordAndAnswer("fish","Omul is a very tasty fish.");
		person_waiting_for_bus.setKeyWordAndAnswer("ice","We'll make a hole into the ice and fish through it.");
		
		person_waiting_for_bus2 = new Chatter("Ruslan","He doesn't seem worry... he'll fix it earlier or later.");
		person_waiting_for_bus2.setKeyWordAndAnswer("long","Oh, some time... I'll fix it... this often happens...");
		
		busdriver1 = new Driver("Sergej","He has been bus driver for all his life.");
		
		busdriver2 = new Driver("Oleg","He has been bus driver for all his life.");
		
		busdriver3 = new Driver("Ivan","He has been bus driver for all his life.");
		
	}


    /**
	 * in this metod the player is provided with all the attributes that relate to him:
	 * the initial items he posesses and the amount of money as well as the time of day,
	 * his initial levels of freezing, hunger and thirst
	 * 
	 */
	private void equipPlayer() {
		player = new Player();
		player.setWear(gloves.getDescription(),gloves);
		player.setCarry(suitcase.getDescription(),suitcase);
		player.setCarry(money.getDescription(),money);
		player.setCarry(camera.getDescription(),camera);
		player.setMoney(65);
		player.setTime(9);
		player.setFreeze(0);
		player.setHunger(0);
		player.setThirst(0);

	}
	
    /**
	 * in this method the Items are built. The all must have a description as an obligatory argument.
	 * set nonstandard or remove actions from them
	 * Then the price is attributed to all items that can be bought during the game.
	 * The suitcase is packed with the items it contains at the beginning of the game.
	 * 
	 */
	private void createItems() {
		
		money= new Money("money",0,"Pounds");	
		camera= new Item("camera");
		camera.setPossAct("take photo - you can't do that if you don't possess a");//useful information for player when he clicks on button camera
		newcamera= new Item("camera");
		camera.setPossAct("take photo - you can't do that if you don't possess a");
		suitcase= new Suitcase("suitcase");
		scarf= new WearableItem("scarf");
		shapka= new WearableItem("shapka");
		gloves= new WearableItem("gloves");
		food = new ConsumableItem("food");
		bottle_of_water = new FreezableItem("water");
		bottle_of_water2 = new FreezableItem("voda");
		suitcase= new Suitcase("suitcase");
		taxiticket = new Item("ticket");
		taxiticket2 = new Item("ticket");
		busticket= new Item("ticket");
		busticket2= new Item("ticket");
		busticket3= new Item("ticket");
		keys = new Item("keys");
		keys.removeAction("buy");
		towel= new Item("towel");
		soap= new Item("milo");
		fish= new ConsumableItem("riba");		
		bottle_of_vodka=new DrinkableItem("vodka");
		bottle_of_vodka2=new DrinkableItem("vodka");
		spikeshoes = new Spikeshoes("spike_shoes");
		spikeshoes_mate = new Spikeshoes("spike_shoes");
		dictionary= new Dictionary("dictionary");
		dictionary2= new Dictionary("slovar");
		ticket = new Item("ticket");
		
		suitcase.pack("scarf", scarf);
		suitcase.pack("food", food);
		suitcase.pack("water",bottle_of_water);
		
		dictionary.setPrize(80);
		dictionary2.setPrize(50);
		camera.setPrize(500);
		shapka.setPrize(120);
		taxiticket.setPrize(140);
		taxiticket2.setPrize(30);
		food.setPrize(20);
	    bottle_of_water2.setPrize(30);
	    bottle_of_vodka.setPrize(80);
	    bottle_of_vodka2.setPrize(120);
	    soap.setPrize(15);
	    fish.setPrize(10);    
	    spikeshoes.setPrize(120);
	    busticket.setPrize(15);
	    busticket2.setPrize(15);
	    busticket3.setPrize(10);
	    keys.setPrize(100);
	    ticket.setPrize(70);
	}

	/**
     * Print out the opening message for the player and the initial setting description.
     */
    private void printWelcome()
    {
    	gui.println(currentRoom.getLongDescription());
    	//gui.println("Priviet! You freshly arrived at Irkutsk airport."); //start screen should be enougth to say hallo.
    }

    /**
     * Create all the rooms and add them to the room set, which is used in the constructor.
     * Set a help message to every room.
     * Link the rooms.
     * Give to each room the information about its representing image.
     * Set the persons to the rooms
     * Set the items to the rooms
     * set the photos that can be taken to the rooms
     * define the initial room
     * 
     */
    private void createRooms()
    {
    airport = new Room("in Irkutsk airport arrival hall");
    rooms.add(airport);
    change_office_open = new Room(" better off now. Somebody heard you shout and is now willing to serve you.");
    rooms.add(change_office_open);
    change_office_closed = new Room("at the foreign exchange office in the airport, unfortunately it looks quite closed... but isn't there some light behind the blind?");
    rooms.add(change_office_closed);
    airport_shop = new Shop("in a shop in the airport hall. Do you still need anything for your Sibiria adventure?");
    rooms.add(airport_shop);
    outside1 = new Transport("outside the airport. Brrr! It's -24C! How cold! Some cabs are waiting for passengers.");
    rooms.add(outside1);
    outside2 = new Transport("outside the airport. Some cabs are waiting for passengers.");
    rooms.add(outside2);
    citycenter = new OpenAir("in the city center of Irkutsk");
    rooms.add(citycenter);
    baikal= new OpenAir("at the biggest lake of the earth: Baikal lake");
    rooms.add(baikal);
    banja= new Shop("in a real Russian banja. If you wash here or have_a_sauna, you can warm up after these terrible temperatures outside.");
    rooms.add(banja);
    bus_station= new OpenAir("at the bus station. But unfortunately there is no bus here at the moment.");
    rooms.add(bus_station);
    hotel= new Transport("in the only hotel of the city");
    rooms.add(hotel);
    lenin_monument= new Transport("now alone with the soldier.");
    rooms.add(lenin_monument);
    city_shop= new Shop("in a shop in the city center");
    rooms.add(city_shop);
    fish_shop= new Shop("at a desk with fresh hot omul from the Baikal");
    rooms.add(fish_shop);
    middle_of_nowhere = new OpenAir("somewhere very far from any town... unfortunately the bus broke down.");
    rooms.add(middle_of_nowhere);
    hotel_room = new Room("in your room now. You share it with another European tourist. It would be a good idea to leave your suitcase here.");
    rooms.add(hotel_room);
    bus = new Transport("at the bus now. Get some tickets.");
    rooms.add(bus);
    busback = new Transport("at the bus now. Get some tickets.");
    rooms.add(busback);
    lenin_monument_with_tourists=new OpenAir("in front of the famous Lenin monument. There are a lot of tourists around");
    rooms.add(lenin_monument_with_tourists);
    on_baikal_lake= new OpenAir("on the Baikal lake around your there are wonderful ice constellations.");
    rooms.add(on_baikal_lake);
    middle_of_nowhere_bus = new Transport("lucky, there is a bus arriving! Try to stop it!");
    rooms.add(middle_of_nowhere_bus);
    city_center_shops = new OpenAir("on the main shopping street of Irkutsk.");
    rooms.add(city_center_shops);
    village = new OpenAir("in a little village at the border of the baikal lake");
    rooms.add(village);
    newspaper_shop = new Shop("in the books and newspapers kiosk.");
    rooms.add(newspaper_shop);
    ice_cave = new OpenAir("in front of an ice cave! Wouldn't it be interesting to have a look in there?");
    rooms.add(ice_cave);

    airport.setHelp("Have a look around and try where you can get from here. \nHave a closer look at the items you are wearing and carrying.");
    airport_shop.setHelp("It is a good idea to talk to the seller, ask for the prices and complain. That will make things cheaper. \nBuy the things directly after your complain, otherwise he'll forget about the offer he made.");
    outside1.setHelp("before you take a taxi, first ask the driver how much it is and whether he is willing to bring you to your hotel.");
    outside2.setHelp("It's ok, you can't get better prices than that.");
    change_office_open.setHelp("No problem to change your money now.");
    change_office_closed.setHelp("You should perhaps shout to get some attention. The change woman is just too lazy to work.");
    citycenter.setHelp("You might take a photo of the wonderful opera.");
    banja.setHelp("Talk to the woman at the desk and if you don't understand her, use a dictionary.");
    bus_station.setHelp("You should wait for a bus to come.");
    hotel.setHelp("Talk to the receptionist to know how much a room is and take your keys.");
    lenin_monument.setHelp("As there are no more people around, you can corrupt the soldier (type \"corrupt soldier\").");
    city_shop.setHelp("Always talk to the seller first: ask for the price of the item you want and tell the seller that it is too expensive. \n If you don't know what a word means, look it up in your dictionary.");
    baikal.setHelp("You should perhaps take a photo of the wonderful Baikal lake.");
    fish_shop.setHelp("You can buy vodka and a fish here (buy riba). Don't forget to discuss the price.");
    middle_of_nowhere.setHelp("You can't do anything else than talking to the person and waiting for another bus.");
    hotel_room.setHelp("Talk to your room mate and leave your suitcase here.");
    bus.setHelp("buy a ticket and choose a destination.");
    busback.setHelp("buy a ticket and choose a destination.");
    lenin_monument_with_tourists.setHelp("There are too many people around... wait a bit.");
    on_baikal_lake.setHelp("You can take a photo here.");
    middle_of_nowhere_bus.setHelp("shout_and_wave, then he will stop.");
    city_center_shops.setHelp("Try to talk to the orthodox woman and have a look at the possible exits.");
    village.setHelp("take a photo of one of those lovely wooden houses!");
    newspaper_shop.setHelp("If you already possess a dictionary there is not much you can do here.");
    ice_cave.setHelp("you should really take a photo here!");
    
    airport.setExit("change office", change_office_closed);
    airport.setExit("shop", airport_shop);
    airport.setExit("cabs", outside1);
    change_office_open.setExit("airport arrival hall", airport);    
    change_office_closed.setExit("airport arrival hall", airport);
    airport_shop.setExit("airport arrival hall", airport);
    outside1.setExit("next cab in queue", outside2);
    outside1.setExit("enter the airport building", airport);
    outside2.setExit("back to first cab",outside1);
    outside2.setExit("enter the airport building", airport);
    citycenter.setExit("hotel",hotel);
    citycenter.setExit("bus station",bus_station);
    citycenter.setExit("banja",banja);
    citycenter.setExit("shopping street",city_center_shops);
    baikal.setExit("bus",busback);
    baikal.setExit("explore Baikal lake",on_baikal_lake);
    baikal.setExit("village near Baikal",village);
    village.setExit("fish shop",fish_shop);
    banja.setExit("city center",citycenter);
    bus_station.setExit("city center",citycenter);
    hotel.setExit("city center",citycenter);
    lenin_monument.setExit("shopping street",city_center_shops);
    city_shop.setExit("shopping street",city_center_shops);
    fish_shop.setExit("village near Baikal", village);
    village.setExit("bus",busback);
    village.setExit("Baikal lake", baikal);
    bus.setExit("bus station",bus_station);
    busback.setExit("baikal lake",baikal);
    hotel_room.setExit("hotel entrance",hotel);
    bus.setTicketExit("Baikal lake",baikal);
    bus.setTicketExit("airport",airport);       
    busback.setExit("explore Baikal lake",on_baikal_lake);
    busback.setExit("village near Baikal",village);
    lenin_monument_with_tourists.setExit("shopping street",city_center_shops);
    on_baikal_lake.setExit("coast",baikal);  
    on_baikal_lake.setExit("ice cave",ice_cave);
    ice_cave.setExit("back to ice splits",on_baikal_lake);
    city_center_shops.setExit("Lenin monument",lenin_monument_with_tourists);
    city_center_shops.setExit("city center",citycenter);
    city_center_shops.setExit("magasin",city_shop);
    city_center_shops.setExit("gasieti",newspaper_shop);
    newspaper_shop.setExit("shopping street", city_center_shops);        

    outside1.setTicketExit("Irkutsk city center", citycenter);
    outside1.setTicketExit("Baikal lake", baikal);
    outside2.setTicketExit("hotel",hotel);
    outside2.setTicketExit("Baikal lake", baikal);
    middle_of_nowhere_bus.setTicketExit("Irkutsk",citycenter);
    busback.setTicketExit("city center",middle_of_nowhere); 
    hotel.setTicketExit("hotel room",hotel_room);
    
    change_office_closed.setChange(change_office_open);
    middle_of_nowhere.setChange(middle_of_nowhere_bus);
    lenin_monument_with_tourists.setChange(lenin_monument);
    bus_station.setChange(bus);
    
    
    airport.setImage("airport.jpg");
    airport_shop.setImage("airport_shop.jpg");
    outside1.setImage("outside1.jpg");
    outside2.setImage("outside2.jpg");
    change_office_open.setImage("change_office_open.jpg");
    change_office_closed.setImage("change_office_closed.jpg");
    citycenter.setImage("citycenter.jpg");
    banja.setImage("banja.JPG");
    bus_station.setImage("bus_station.jpg");
    hotel.setImage("hotel.jpg");
    lenin_monument.setImage("lenin_monument.jpg");
    city_shop.setImage("magasin.jpg");
    baikal.setImage("bus_on_ice.jpg");
    fish_shop.setImage("fish_shop.JPG");
    middle_of_nowhere.setImage("TN_middle_of_nowhere.JPG");
    hotel_room.setImage("hotel_room.JPG");
    bus.setImage("bus.jpg");
    busback.setImage("baikalbus.JPG");
    lenin_monument_with_tourists.setImage("lenin_monument_with_tourists.jpg");
    on_baikal_lake.setImage("on_lake.jpg");
    middle_of_nowhere_bus.setImage("middle_of_nowhere_bus_arriving.jpg");
    city_center_shops.setImage("TN_Innenstadt.JPG");
    village.setImage("irkutsk_wooden_houses.jpg");
    newspaper_shop.setImage("newspaper_shop.JPG");
    ice_cave.setImage("ice_cave.jpg");
    
    airport_shop.setPerson(airport_shop_seller);    
    outside1.setPerson(taxidriver);
    outside2.setPerson(taxidriver2);
    change_office_open.setPerson(change_woman);
    banja.setPerson(banja_woman);
    bus_station.setPerson(person_waiting_for_bus);
    hotel.setPerson(hotel_waiter);
    lenin_monument.setPerson(soldier);
    city_shop.setPerson(city_shop_seller);
    fish_shop.setPerson(fish_shop_seller);
    middle_of_nowhere.setPerson(person_waiting_for_bus2);
    hotel_room.setPerson(tourist);
    bus.setPerson(busdriver1);
    busback.setPerson(busdriver2);
    lenin_monument_with_tourists.setPerson(soldier);
    middle_of_nowhere_bus.setPerson(busdriver3);
    city_center_shops.setPerson(orthodox_woman);
    newspaper_shop.setPerson(newspaper_woman);
    airport_shop.setItem(dictionary.getDescription(), dictionary);
    airport_shop.setItem(shapka.getDescription(), shapka );
    airport_shop.setItem(newcamera.getDescription(), camera);  
    outside1.setItem(taxiticket.getDescription(),taxiticket);
    outside2.setItem(taxiticket2.getDescription(),taxiticket2);
    city_shop.setItem(food.getDescription(),food);
    newspaper_shop.setItem(dictionary2.getDescription(),dictionary2);
    city_shop.setItem(bottle_of_water2.getDescription(),bottle_of_water2);
    city_shop.setItem(bottle_of_vodka.getDescription(),bottle_of_vodka);
    city_shop.setItem(soap.getDescription(),soap);
    fish_shop.setItem(fish.getDescription(),fish); 
    fish_shop.setItem(bottle_of_vodka2.getDescription(),bottle_of_vodka2);
    hotel_room.setItem(towel.getDescription(),towel);
    city_shop.setItem(spikeshoes.getDescription(),spikeshoes);
    bus.setItem(busticket.getDescription(),busticket);
    busback.setItem(busticket2.getDescription(),busticket2);
    hotel.setItem(keys.getDescription(),keys);  
    banja.setItem(ticket.getDescription(),ticket);
    
    on_baikal_lake.setPhoto("icesplits.jpg");
    citycenter.setPhoto("dram1.jpg");
    village.setPhoto("wooden_house.jpg");
    baikal.setPhoto("baikal.jpg");
    ice_cave.setPhoto("photo_out.jpg");
    
    currentRoom = airport;
    
    }
  

	/**
     * Given a command, process the command. That is call the methods to
     * execute the command and give to them the command elements they need.
     * 
     */
    public void interpretCommand(String commandLine) 
    {	
    	Command command = parser.getCommand(commandLine);//parse the command
    	System.out.println("your action: " + commandLine);//makes it easier for the corrector to follow what happens, so I leave it
        String commandWord = command.getCommandWord();//split the command into commandWord and
        String commandArgument = command.getArgument();//commandArgument(the rest of the command)
        if (commandWord.equals("help")){
            printHelp();
        }        
        else 
        	if (commandWord.equals("go")){
        	go(commandArgument);    
        }
        else if (commandWord.equals("shout_and_wave")) {       	
        	shoutWave();
        	}
        else if (commandWord.equals("wait")){
        	waitaction();
        	}
        else if (commandWord.equals("quit")) 
            endGame();   
        else if ((commandWord.equals("wash")|| commandWord.equals("have_a_sauna"))){
        	wash();
        }
        else if (!(commandArgument == null)){//if it was a at-least-two-component command
        	if (commandWord.equals("take")){
            	take(commandArgument);
            }
        	else if (commandWord.equals("buy")&&!(commandArgument==null))   {  
            	buy(commandArgument);
            }
            else if ( commandArgument.equals("money")&& commandWord.equals("change")){            		
            	changeMoney();            		
            }
            else if (commandWord.equals("open")){
            	open(commandArgument);
            }
            	         		
            else if ((commandWord.equals("eat")||commandWord.equals("drink")||commandWord.equals("put_on"))){
            	consume(commandArgument);
            }       
            else if (commandWord.equals("look_up")){
            	look_up(commandArgument);
            }            
            else if (commandWord.equals("take_off")){
            	take_off(commandArgument);
            }
            else if (commandWord.equals("corrupt")){
            	currupt(commandArgument);
            } 
            else {//handling of two-or-three-components commands: //this does not work when the second argument is a compositum 
            	Command command2 = parser.getCommand(commandArgument);//parse the rest of the command
        		String commandWord2 = command2.getCommandWord();
        		String commandArgument2 = command2.getArgument();
            	if ((commandWord.equals("put"))){                	
                	put(commandWord2,commandArgument2);
                }
                else if ((commandWord.equals("talk"))&& !(commandArgument == null)){
                	talk(commandWord2,commandArgument2);
                }                
                else if ((commandWord.equals("apply")&&!(commandWord2== null)&&!(commandArgument2==null))){//should only be invoked when user presses an item button
            		apply(commandWord2,commandArgument2);
                }
                else gui.println("What do you mean???");//in case it was no valid or an uncomplete several-words command
            }
        }
        else if (commandWord.equals("talk_to_person")){
        	talk_to_person(commandWord,commandArgument);
        }	     	
        else gui.println("What do you mean???");//in case it was no valid or an uncomplete one-word command
        
        gui.update();//after each action update the userInterface to actualize the situation representation        
        if (outOfGame()){//control whether the player has already lost or won the game and react appropriately
        	//gui.disposemain();//if that is the case, close userinterface
        	gui.makeInvisible();
        }
    }
    
    /**
     * open suitcases and print error for all other items.
	 * @param commandArgument item which is to be opened
	 */
	private void open(String commandArgument) {
		if( commandArgument.equals("suitcase")){
        	gui.println(currentRoom.openSuitcase(suitcase)); 
    	}
    	else gui.println("You can not open this item.");
		
	}



//**********************implementations of user commands:**************************************

	/**
	 * test whether there is a person in the room, in case of failiure 
     * give that information to the user.
     * answer to the utterance made by the user and perform the special events
     * which occur when talking to particular persons.
	 * @param commandWord2 name of the person with whom the user wishes to talk
	 * @param commandArgument2 utterance he wishes to communicate
	 */
	private void talk(String commandWord2, String commandArgument2) {
		if (currentRoom.HasPerson()){
    		if (commandArgument2.endsWith("close_dialogue")){// for the case in which the dialog window was used
    			gui.println("dialogue closed");//confirm action to the player
    		}
    		else{
    			if (currentRoom.getPerson()== tourist&&spike_shoes_given==false){//special event for when you talk to the tourist
        			currentRoom.setItem("spike_shoes",spikeshoes_mate);//spike shoes are then a new item in the room where the tourist is.
        			spike_shoes_given=true;
    			}
        		else if (currentRoom.getPerson()== orthodox_woman){//special event for when you talk to the orthodox woman 
        			currentRoom.setPhoto("XPAM.jpg");//simulates that she makes the photo as a present to the player. The photo is not available in the room before.
        			takePhoto();//this action is normally performed, when the user takes a photo.
        		}
        		Person p = (Person) currentRoom.getPerson();
        		String n = p.getName();
        		if (commandWord2.equals(n)){//if the player typed the correct name for the person in the actual room        	
        			gui.println(p.getName() + ": " + p.reply(commandArgument2));//print the reply to the command to the user interface text output area
        			}
        		else gui.println("The name of the person in this room is "+ n +".");//tell the player the correct name or the person in the room.
    		}        	
    	}
    	else gui.println("Sorry, but there is nobody here to whom you could talk.");	
	}

	
    /**
     * test whether there is a person in the room, in case of failiure 
     * give that information to the user.
     * get an utterance from the player if not provided
     * then answer to an utterance made by the player
	 * @param commandWord
	 * @param commandArgument
	 */
	private void talk_to_person(String commandWord, String commandArgument) {
		if (currentRoom.HasPerson()){
    		if(commandArgument == null){//if the talk_to_person button from the interface is used, there is no utterance specified
        		gui.dialogWindow();//then the dialog window is called to get this argument from the user.
        	}
    		else //the user might type into the input text field "talk_to_person bla bla bla". 
    			gui.println(currentRoom.getPerson().reply(commandArgument));//get the answer to bla bla bla.
    	}
    	else gui.println("Sorry, but there is nobody here to whom you could talk.");		
	}



	/**
	 * print out the information about the actions available for the demanded item.
	 * @param commandWord2
	 * @param commandArgument2
	 */
	private void apply(String commandWord2, String commandArgument2) {
		if (commandWord2.equals("item_in_room")){
			//System.out.println(((Item) getItems().get(commandArgument2)).getDescription());
			gui.println(((Item) getItems().get(commandArgument2)).getPossActDescr());
		}
		else if (commandWord2.equals("worn")){
			gui.println(((Item) player.getWorn().get(commandArgument2)).getPossActDescr());
		}
		else if (commandWord2.equals("carried")){
			gui.println(((Item) player.getCarried().get(commandArgument2)).getPossActDescr());
		}	
	}


	/**
	 * put down an item (that is change it's location from carried by the player
	 * to the room. In case there is a third argument and this third argument is 
	 * suitcase put the item specified in second argument(all but the suitcase itself and the money) into the suitcase 
	 * (in case the user has a suitcase in reach)
	 * @param commandWord2 carried item
	 * @param commandArgument2 only suitcase possible
	 */
	private void put(String commandWord2, String commandArgument2) {
		if (commandArgument2== null){   
    		gui.println(currentRoom.put(commandWord2));
			}
		else if (commandArgument2.equals("suitcase")&& (getItems().containsValue(suitcase))||player.getCarried().containsValue(suitcase)){
			if (commandWord2=="suitcase"||commandWord2=="money"){
				gui.println("This does not work.");
			}
			else{
				suitcase.pack(commandWord2,(Item) getItems().get(commandWord2));
				getItems().remove(commandWord2);
				player.remove(commandWord2);//to allow the player to put carried as well as non carried things into the suitcase
			}
		}	
		else gui.println("Sorry, you can't put "+commandWord2+"s into a"+commandArgument2+".");//give feedback to player in case it did not work
	}
	
	/**
	 * change location of item from carried to in room
	 * @param commandArgument
	 */
	private void put(String item) {
		currentRoom.setItem(item,(Item) player.getCarried().get(item));
		player.remove(item);
		
	}


	/**
	 * very specialized method, only works with argument "soldier" and in room "lenin monument"
	 * otherwise gives feedback to the user that it was not possible to perform the action
	 * player pays fixed sum for having the photo available
	 * in the lenin_monument room
	 * @param commandArgument soldier
	 */
	private void currupt(String commandArgument) {
		if (commandArgument.equals("soldier")&& currentRoom==lenin_monument){
    		player.setMoney(player.getMoney() -  200);//soldier wants 200 roubles
    		currentRoom.setPhoto("Lenin monument.jpg");//make photo available
    		gui.println("You corrupted the soldier. Try again to take a photo.");
    	}
    	else gui.println("I'm sorry, you can't corrupt this person here.");
	}



	/**
	 * change location of wearable items from "worn" to "in room".
	 * if preconditions not appropriate gives that info to player and does nothing
	 * @param commandArgument the description of a worn item
	 */
	private void take_off(String commandArgument) {
		if(player.getWorn().containsKey(commandArgument)){    //precondition: player wears item    		
    		player.getCarried().put(commandArgument,player.getWorn().get(commandArgument));//put it into room
			player.getWorn().remove(commandArgument);//remove it from set of worn items
			gui.println("You put off your " + commandArgument);//confirmation to player
    	}
    	else gui.println("You can't put off things you are actually not wearing.");		
	}



	/**
	 * looks up a Russian word in the dictionary and gives back its translation to English 
	 * @param commandArgument should be a Russian word
	 */
	private void look_up(String commandArgument) {
		if (player.getCarried().containsValue(dictionary)||player.getCarried().containsValue(dictionary2))
    		gui.println("Result from looking up \""+commandArgument+ "\" in your dictionary: " +dictionary.look_up(commandArgument));//msste eigentlich je nachdem in dictionary 1/2 nachsehen
    	else gui.println("You need a dictionary to look up things.");		
	}



	/**
	 * specialized command, only works in banja, when user has both towel and soap.
	 * otherwise gives back a hint how to wash.
	 * decreases freezing by 10 points.
	 * 
	 */
	private void wash() {
		if (currentRoom == banja){
			if (player.getCarried().containsKey("ticket")){
				if (player.getCarried().containsValue(towel)&& player.getCarried().containsValue(soap)){//player.getCarried().containsKey("towel")&& player.getCarried().containsKey("milo")){
		    		player.setFreeze(-10);
		    		gui.println("You sauned in a real Russian banja, were hit with birk tree branches and really enjoyed it. Furthermore you warmed up now.");
		    	}
		    	else gui.println("This does not work at the moment... ask"+ currentRoom.getPerson().getName()+".");		
				}
			else gui.println("You have to buy a ticket before you can saune.");
			}
		else gui.println("If you want to wash or have a sauna you have to go the banja.");
	}



	/**
	 * permits the player to change the location of an item from in the room to carried by him.
	 * if the commandArgument is photo a photo is taken(calls takePhoto()) 
	 * in case this is possible for that room,
	 * @param commandArgument any item in room, special if "photo"
	 */
	private void take(String commandArgument) {
		if (commandArgument.equals("photo")){
    		if (currentRoom.photoavailable()&& !(currentRoom==city_center_shops) && player.getCarried().containsKey("camera")){        			
    			takePhoto();
    			} 
    		else gui.println("It is not possible to take a photo here.");
    	}
		
    	else if (commandArgument.equals("keys")){
        		if(currentRoom==hotel){//get the keys, which have a price and without which you can't access your room(similar to ticket) without paying anything for them, this design makes you lose the game, if you even can't pay it for one night
        			int m=player.getMoney();//get actual amount of money possessed by player
        			interpretCommand("buy keys");
        			player.setMoney(m);//set Player back to old money amount, he will pay then for every night, not for the keys
        			gui.println("Every night in the hotel is 300 Roubles.");
        		}
        	}
    	else gui.println(currentRoom.take(commandArgument));//for example: take suitcase		
	}



	/**
	 * tests whether applicable and in that case performs one of eat, drink, put_on
	 * as appropriate.
	 * @param commandArgument should only carried consumable or wearable items
	 */
	private void consume(String commandArgument) {
		if (player.getCarried().containsKey(commandArgument)){//only for carried items
    		Item i = (Item) player.getCarried().get(commandArgument);//get the item, not only its description
    		gui.println(i.consume(player,currentRoom));//special method performed dependent on class of item
    	}
    	else if (currentRoom.getItems().containsKey(commandArgument)){
			gui.println("You must take this items first.");}
		else gui.println("This item is not available to you at the moment.");		
	}



	/**
	 * decreases the money of the player by the prize of the item
	 * changes item location from item in room to carried item 
	 * @param String item to be bought
	 */
	private void buy(String commandArgument) {
		String player_currency = player.getCurrency();
    	if (currency.equalsIgnoreCase(player_currency)){//tests whether currencies correspond to each other
    		gui.println(currentRoom.buy(commandArgument));//procedure of buy depends on room (transport vs shop), also handels case where item not available
    	}
    	else gui.println("We don't take "+ player_currency + " here. You have to pay in " + currency +".");
   
	}



	/**
	 * tests the conditions for winning or loosing the game and 
	 * in case of decided outcome invokes interface to close game display and 
	 * display final screen
	 * @return value saying whether the outcome of the game is already decided or not.
	 */
	private boolean outOfGame() {
		if (player.getFreeze()>maxfreeze){//max freezing level exceeded
			gui.showPhoto("You lost! You are becoming similar to a block of ice.","frozen.jpg","ok, next time I'll cloth warmer or use the banja");
			game_over=true;
		}
		else if (player.getThirst()>maxthirst){//max thirst level exceeded
			gui.showPhoto("You did not manage to drink enough.","drink.jpg","ok, next time I'll drink more");
			game_over=true;
		}
		else if (player.getMoney()<=0){//no money left
			gui.showPhoto("You ran out of money because you bought something that was more expensive than you could afford you your money was stolen.","no_money.jpg","ok, next time I will take more care about my money");
			game_over=true;
		}
		//game won!
		else if (photos==7 &&((Money)player.getCarried().get("money")).getCurrency().equals("")){
			if(!(hotel_room.getItems().containsKey("spike_shoes"))&& spike_shoes_given){
				gui.println("You forgot to give back the spike shoes you let from your room mate.");
				
			}
			else if(!(hotel.getItems().containsValue(keys))){
				gui.println("You forgot to give back the keys to the hotel reception.");
				
			}
			else if(!(player.getCarried().containsValue(suitcase))){
				gui.println("Where is your suitcase???");
				
			}
			else
			{
				gui.showPhoto("You won!!! You became almost a real Russian voyager!","man_with_suitcase.JPG","Hurray! I'll travel to Sibiria in my real life!");
				game_over=true;
			}
		}		
		return game_over;//game goes on
	}



	/**
	 * counts up photo count if *new* picture taken and calls photo-show method in from user interface
	 * is called by other method, only if photo available in that room, so this is not tested.
	 *
	 */
	private void takePhoto(){
    	if (!(currentRoom.photoAlreadyTaken())){//don't count a photo twice
			photos = photos + 1;//photo counter (max 7)
			if (photos == maxphotos) {
				gui.println("Now you make any steps necessary to fly back home.");//Give the user a hint about what he has to do to win the game.
			}
			gui.println("You took a photo.");
    	}    	
		else gui.println("You already took this photo.");        			
		gui.showPhoto("photo",currentRoom.photo(),"continue game");//show Photo for user (popup window)
    }

	
	/**
	 * change money from Pounds to Roubles, Roubles to Euros or Euros to Roubles
	 * by manipulating money amount and changing currency.
	 * precondition: player is in the opened change office
	 */
	private void changeMoney() {
		if (currentRoom == change_office_open){
			Money m = ((Money)player.getCarried().get("money"));
			String c = m.getCurrency();
			String nc="";
			int moneya=0;
			if ( c.equals("Pounds")){//get currency value from item money (class Money)
				moneya=player.getMoney() * 44;//1:44 is the exchange rate
				player.setMoney(moneya);//1:44 is the exchange rate
				nc="Roubles";
				m.setCurrency(nc);//change currency in item money 
				
			}
			else if (c.equals("Roubles")){
				moneya = player.getMoney()/31;
				player.setMoney(moneya);//31:1 is the exchange rate				
				nc= "";
				m.setCurrency(nc);
			}
			else if (c.equals("")){
				moneya=player.getMoney()*31;
				player.setMoney(moneya);//1:31 is the exchange rate
				nc = "Roubles";
				m.setCurrency(nc);
			}
			gui.println("You changed your money and now have " + moneya + " "+nc + "." );
		}
		else gui.println("You can't change money here.");
	}



	/**
	 * wait for a random amount of time (time is update)
	 * if you wait outside, you will freeze depending on how warm you are clothed
	 * in case this is possible for that room, the actual room is set to its changed situation
	 * 
	 */
	private void waitaction() {
		int i = randomNumber.nextInt(5)+2;//get amount of time you have to wait
		currentRoom.timeUpdate(i);//change time (also increases hunger and thirst)
		HashMap h = player.getWorn();
		if (currentRoom.isRoom().equals("OpenAir")){
			if (h.containsValue(gloves)&& h.containsValue(scarf)&&h.containsValue(shapka)){//gloves,scarf and shapka are the warming items
				i=i/2;//if the player is warmly clothed he freezes less
			}
			if (player.getHunger()<0){
				i=i+1;//if the player is hungry, he freezes more.
			}
			player.setFreeze(i);
		}
		if (currentRoom.changeable() ){
			currentRoom= currentRoom.changeSit();//change actual room to defined changed situation of actual room
			gui.println(currentRoom.getLongDescription());			
		}
	}


	/**
	 * shout and wave to get somebodies attention-
	 * works for opening the change office and stopping the bus in the middle of nowhere
	 * (both of these are change-situations)
	 * 
	 */ 
	private void shoutWave() {
		if (currentRoom==change_office_closed){//to open change office
				currentRoom = currentRoom.changeSit();//the actual situation becomes the new situation
				gui.println(currentRoom.getLongDescription());//give information about the new situation
		}
		else if (currentRoom==middle_of_nowhere_bus){//to stop bus
			middle_of_nowhere_bus.setItem(busticket3.getDescription(),busticket3);//make ticket available to be bought
			gui.println("The bus stopped. Now you can buy another bus ticket.");
		}
		else gui.println("Shouting and waving does not have any effects here.");
	}

	
	/**
     * Print out some help information, stored in each room
     */
    private void printHelp() 
    {	
    	gui.println(currentRoom.getHelp());    	
    }

    /** 
     * Try to go to a new room. If there is an exit, enter the new
     * room, otherwise print an error message.
     * Time goes by when rooms are changed.
     * Special condition of wearing spike shoes only in outdoor places is handled.
     * Special condition for beginning of new day after entering hotel room in the evening is handled
     */
    private void go(String nextLoc) 
    {
    	if ((player.getWorn().containsKey("spike_shoes") && currentRoom.isRoom().equals("OpenAir"))|| !(player.getWorn().containsKey("spike_shoes"))){
    		if (nextLoc==null){//if the information is incomplete, 
    			nextLoc = gui.directionDialog(currentRoom.getExits());//get it through a direction dialog proposing the different possibilities
    		}    	
    		if (currentRoom==baikal &&!(player.getWorn().containsKey("spike_shoes")) && currentRoom.getExits().get(nextLoc)==on_baikal_lake){
    			//in this special situation you have to wear spike shoes. The go command is not executed.
    			gui.println("For this you would have to walk on the ice. But you can't ... it's too slippery.");
    		}		
    		else if (currentRoom.getExit(nextLoc)==currentRoom){
    			gui.println("Sorry. This room does not have such an exit.");//error message if room does not have specified exit
    			}
    		else {
    			int time = getTime();//get the actual time and store it in variable time
    			if (currentRoom.getExit(nextLoc)==hotel_room && time>=17){//if you enter your hotel room after 5pm
    				player.setTime(9);//set the actual time to 9 am    				
    				player.setHunger(((player.getHunger())*(-1)));//set the actual hunger of player to 0
    				player.setThirst(((player.getThirst())*(-1)));//set the actual thirst of player to 0
    				player.setFreeze(((player.getFreeze())*(-1)));//set the actual freezing level of player to 0
    				player.setMoney(player.getMoney()-((Item)player.getCarried().get("keys")).getTouristPrize());//make the player pay for the night
    				gui.println("you slept, recovered and had a wonderful breakfast. The new day can start right now!");
    				currentRoom= currentRoom.getExit(nextLoc);
    			}
    			
    			else{
    				currentRoom.timeUpdate();//changing the location is time-consuming: time goes by depending on room (2 time units for open air rooms, 1 for all others
        			if (currentRoom.isRoom().equals("Transport")&&!(currentRoom==hotel)){
        				if (player.getCarried().containsKey("ticket")){//if the player bought a ticket and now goes somewhere from this room, the ticket is taken from him and put back into the room.
        					put("ticket");
        					Room lastRoom = currentRoom;
        					currentRoom = currentRoom.getExit(nextLoc); //change actual room    	
                			gui.println(currentRoom.getLongDescription());//print info message about new room for player    			
        					((Transport) lastRoom).removeTicketExits();
        				}  
        				else {
        					currentRoom = currentRoom.getExit(nextLoc); //change actual room    	
                			gui.println(currentRoom.getLongDescription());//print info message about new room for player    			
        				}
        			}
        			else
        			{
        				currentRoom = currentRoom.getExit(nextLoc); //change actual room    	
            			gui.println(currentRoom.getLongDescription());//print info message about new room for player    			
        			}
    			}
        			
    			
    			
    		}
    	}
    	else gui.println("You must not walk here with spike shoes."); //if player wears spike shoes and the room is not an open air room       	
    }
        

    
    /**
     * if player quits game before he has lost or won it
     * bye-message is printed to textual output and user interface is closed.     *
     */
	private void endGame()
    {
        gui.println("Hope Sibiria didn't scare you? It's a wonderful region. See you again soon!");
        gui.close();
    }

	
	/**
	 * calls a method in the userinterface that makes the userinterface visible to the user
	 *
	 */
	public void setVisible() {
		gui.setmainVisible();
		
	}




	/**
	 * accessor method that gives back time of the day
	 * and handles the risk of being robbed after 7 pm
	 * @return int time in hours
	 */
	public int getTime() {
		int	time = player.getTime();
		if (time > 19){	//risk of robbery implementation:	
			gui.println("be careful, you could get robbed during the night!");
			float rn = randomNumber.nextFloat();//you may have good or bad luck: a random number is generated 
			 rn = rn/5*(24-time);//the probability of being robbed increases the later it is...
			 if (rn*rn > 0.4){//...nonlinearly; if this condition is true, the player is going to be robbed
			 	HashMap carried = player.getCarried();//one of his carried items
			 	Set keys = carried.keySet();
			 	int k = keys.size();
			 	int steal = randomNumber.nextInt(k);//randomly one of the items is chosen
			 	Iterator it = keys.iterator();
			 	int j=0;
			 	String lostItem = "nothing";
			 	while (it.hasNext() && j<= steal){
			 		lostItem=(String)it.next();			 		
			 		j=j+1;
			 	}
			 	carried.remove(lostItem);//the chosen item is removed from the player
			 	gui.println("Your " + lostItem + " has been stolen from you!");// and he is informed.
			 }
		}		
		return time;//return current time
	}

	//**********************some more accessor methods********************************
	/**
	 * returns items of curren room
	 * @return a Hashmap of the items contained in the current Room
	 */
	public HashMap getItems() {
		
		return currentRoom.getItems();
	}

	/**
	 * retrieve the image of the acutal room
	 * @return String imageaddress saved in room
	 */
	public String getImage() {
		return currentRoom.getImageID();		
	}
	
	/**
	 * accessor method for the number of actually taken photos
	 * @return the number of actually taken photos stored in field variable photos
	 */
	public int getPhotos(){
		return photos;
	}


	/**
	 * accessor method for Player player
	 * @return the player
	 */
	public Player getPlayer() {
		return player;
	}

	/**
	 * accessor method for Room currentRoom
	 * @returns the current room 
	 */
	public Room getCurrent() {
		return currentRoom;
	}
	
	


	/**
	 * @return
	 */
	public int getMaxFreeze() {
		
		return maxfreeze;
	}



	/**
	 * @return
	 */
	public int getMaxThirst() {
		
		return maxthirst;
	}



	/**
	 * @return
	 */
	public int getMaxPhotos() {
		
		return maxphotos;
	}


	/**
	 * @return
	 */
	public boolean gameOver() {
		return game_over;
	}
	


	
}

