/*
 * Created on 11.12.2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package sibiriaGame;


import java.util.HashMap;

/**
 * This class is a part of the "Travelling in Sibiria" application.
 * I represents the player and his actual attributes.
 * 
 * @author Vera Demberg
 * @version 1.0 (December 2004)
 */
public class Player {
	
	private HashMap carried;//items carried by player
	private HashMap worn;//items worn by player
	private int freeze;//player's level of freezing
	private int hunger;//player's level of hunger
	private int thirst;//player's level of thirst
	private int money_amount;	//player's amount of money
	private int time;//actual time of the day for the player
	private int timeUnit = 3;//time units per hour in which steps are counted
	private int oldtime;//time in hours not time units before actual change of time.
	
	
	/**
	 * Initialize the carried and worn things for the player
	 * set all attributes to 0 (not money because money is an item on its own and not only an attribute)
	 */
	public Player(){
		carried = new HashMap();
		worn = new HashMap();
		freeze=0;
		hunger=0;
		thirst=0;
		time = 0;
	}


	/**
	 * updates the time by a given number of time units and
	 * stores the time it was before the update
	 * changes the amount of hunger and thirst if an hour has passed
	 * @param i the number of time untis that goes by
	 */
	public void timeUpdate(int i) {
		oldtime = time/timeUnit;	//store old time in hours (not time units)	
		time=((time + i)%(24*timeUnit));//calculate new time in time units
		if (!(oldtime==time/timeUnit)){//if a whole hour has passed 
			setHunger(1);//increase hunger and thirst by 1
			setThirst(1);
		}	
	}	
	
	/**
	 * set the time to a new value i in hours
	 * @param int i, time in hours
	 */
	public void setTime(int i) {
		time = i*timeUnit;//because of different possible timesteps
		oldtime=i;//that information is needed for hunger/thirst-handling in updateTime method
		
	}

	/**
	 * look at the money item if possessed by player to determine how much player possesses
	 * @return the amount of money possessed by player
	 */
	public int getMoney() {
		if (getCarried().containsKey("money")){
			return ((Money) getCarried().get("money")).getMoney();
		}
		else return 0;//if money was stolen or put somewhere
	}
	
	/**
	 * get actual time in hours
	 * @return int time in hours
	 */
	public int getTime() {
		return time/timeUnit; //time is counted in not-hourly-minutes steps, but this is not shown to the user.
	}
	
	/**
	 * put an item to the set of worn things by player
	 * @param description of item
	 * @param item itself
	 */
	public void setWear(String descr,WearableItem item) {
		worn.put(descr,item);		
	}
	
	/**
	 * 
	 * @return HashMap of fall worn items
	 */
	public HashMap getWorn(){	
		return worn;
	}
	

	/**
	 * removes an item either place where the player has it (worn and carried)
	 * @param item to be removed
	 */
	public void remove(String item) {
		carried.remove(item);
		worn.remove(item);
		
	}

	/**
	 * increase or decrease amount of posessed money
	 * @param i: amount of money, increases richness of player if negative
	 */
	public void setMoney(int i) {
		((Money)getCarried().get("money")).setMoney(i);
		//get Information from money-Item and add i to old amount, store in item money
		
	}

	

	/**
	 * accessor method for currency, in which the player's money is
	 * if money item does not exist returns an empty String (e.g. if money stolen)
	 * @return String currency (for example "Pounds"
	 */
	public String getCurrency() {
		if (getCarried().containsKey("money")){
			return((Money)getCarried().get("money")).getCurrency();//get Information from money-Item
		}
		else return "";
	}
	//********************other various accessor and mutator methods*********************
	
	/**
	 * increase hunger by i
	 * @param i
	 */
	public void setHunger(int i) {
		hunger=hunger + i;
		
	}
	
	/**
	 * increase thirst by i
	 * @param i
	 */
	public void setThirst(int i) {
		thirst = thirst + i;
		
	}

	/**
	 * increase freezelevel by i
	 * @param i
	 */
	public void setFreeze(int i) {
		freeze = freeze+ i;
		
	}
	/**
	 * add an item to the carried things
	 */
	public void setCarry(String descr,Item item) {
		carried.put(descr,item);
		
	}
	
	/**
	 * 
	 * @return carried things
	 */
	public HashMap getCarried(){
		return carried;
	}
	/**
	 * @return int freezing level
	 */
	public int getFreeze() {
		
		return freeze;
	}

	/**
	 * @return int hunger
	 */
	public int getHunger() {
		
		return hunger;
	}


	/**
	 * @return int thirst
	 */
	public int getThirst() {
		
		return thirst;
	}


	
}
