/*
 * Created on 11.12.2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package sibiriaGame;
import java.util.Set;
import java.util.HashMap;
import java.util.Iterator;


/*
 * Class Room - a room in an adventure game.
 *
 * This class is part of the "Travelling in Sibiria" application.  
 *
 * A "Room" represents one location in the scenery of the game.  It is 
 * connected to other rooms via exits.  For each existing exit, the room 
 * stores a reference to the neighboring room.
 * 
 * Subclasses are: Shop, Transport, OpenAir
 * @author  Vera Demberg
 * @version 1.0 (December 2004)
 */

public class Room 
{
    private String description;//stores description of room
    private HashMap exits;// stores exits of this room.
	private HashMap items;//stores items that are in room
	private String imageident;//stores path to picture representing room
	private Room newSit;//stores exit to Room available through actions like waiting/shout_and_wave 
	private boolean changable = false;// stores whether such an exit exists for a room, default:false, has to be explicitly set to true
	private String help;//stores help text for room
	private Person person;//stores Character contained in room	
	private boolean roomHasPerson = false; //whether the room does contain a person at all
	private String photoid;//stores location of photo that can be taken in room 
	private boolean photoTaken = false;//whether the photo of this room has already been taken
	private boolean photoavailable=false;//whether a photo can be taken in this room
	protected Player player;//also used by subclasses

    /**
     * Create a room described "description" with a given image. 
     * Initially, it has no exits. "description" is something like 
     * "in a kitchen" or "in an open court yard".
     */
    public Room(String description) 
    {
        this.description = description;
        exits = new HashMap();
		items = new HashMap();
    }
    
    
	
    /**
     * Define an exit from this room.
     */
    public void setExit(String nldescription, Room neighbor) 
    {    	
        exits.put(nldescription, neighbor);
    }
    
    /**
     * define an exit from this room that is only availabla under certain conditions (after waiting/shout_and_waiting)
     * each room has at most one change room. does not need a string parameter because not shown to user for free choice
     * @param changeroom the room which can be acessed by a change
     */
    public void setChange(Room changeroom) 
    {
        newSit= changeroom;
        changable = true;       
    }

    /**
     * Return the description of the room (the one that was defined in the
     * constructor).
     */
    public String getShortDescription()
    {
        return description;
    }

    /**
     * Return a long description of this room, in the form:
     *     You are in the kitchen.
     * 		[The person in this room is Peter.]
     * 		[There are the following items here: bike, tin]
     *     Exits: north, west
     */
    public String getLongDescription()
    { 	
    	String longdescr = "You are " + description;//desciption of room
    	if (HasPerson()){//description of person in room if there is one
    		longdescr = longdescr + "\nThe person here is "+ getPerson().getName() +". " + getPerson().getDescription();
    	}
    	Set items = getItems().keySet();//iterate over items in room
    	Iterator i = items.iterator();
    	String idescr = "";
    	while(i.hasNext()){
    		idescr = idescr + i.next() + "\t";//add all items
    	}
    	if (!(idescr=="")){
    		longdescr = longdescr + "\n"+"There are the following items here:\n" + idescr;//and an explanation for the list of items
    	}
    	longdescr = longdescr+"\n"+ getExitString();//add information about available exits
        return  longdescr;
    }

    /**
     * Return a string describing the room's exits, for example
     * "Exits: north, west".
     */
    protected String getExitString()
    {
        String returnString = "Exits: ";
        Set keys = exits.keySet();
        for(Iterator iter = keys.iterator(); iter.hasNext(); )//iterate over all accessible rooms
            returnString += iter.next()+", " ;
        returnString = returnString.substring(0,returnString.length()-2);//cut off the final ", " or the ": "(if no exits there)(can happen if only ticketExits available, these are only added later in the game, not at instanciation time)
        return returnString;
    }

    /**
     * Return the room that is reached if nldescription (this could e.g. be "kitchen" for room kitchen) of it was demanded
     */
    public Room getExit(String nldescription) {
     if (exits.containsKey(nldescription)){
        return (Room)exits.get(nldescription);}
    else 
    	return this;
    
    }
    /**
     * get the items contained in this room
     * and let them be sold by the person in this room,
     * that means: create one sentence per item for the language set of that person
     * 
     *
     */
    public void initializePerson(){
    	if (!(getItems().isEmpty())){
    	getPerson().setSoldItems(getItems());
    	}
    }
    
	/**
	 * open the suitcase and take the items out of it to put them into the room
	 * this method is part of the Room class, because it is overridden by a method
	 * from the Shop class (subclass of Room)
	 * 
	 * @param Suitcase suitcase, a special type of Item
	 * @return a confirmation of what has been done
	 */
	public String openSuitcase(Suitcase suitcase) {
		HashMap unpacked=suitcase.unpack();
		Set keys = unpacked.keySet();//get items that are contained in suitcase
		System.out.println(keys);
		Iterator iter = keys.iterator();
		while (iter.hasNext()){
			String i=(String) iter.next();
			setItem(i,(Item) unpacked.get(i));//put them into the room
		}
		suitcase.emptysuitcase();
		return "You took all the items that were in it out of the suitcase.";
	}
    
	/**
	 * remove an item from the things contained in room and move it to
	 * the set of things carried by player
	 * (in this class because overridden by method in shop class)
	 * @param description of item
	 * @return String: a confirmation
	 */
	public String take(String item) {
		if (getItems().containsKey(item)){
			//if (!(player.carryconstraintExceeded(item))){//TODO I did not have time to implement the carry-constraint
		
				player.setCarry(item,(Item)getItems().get(item));//put item to carried items
				removeItem(item);//remove it from room
				return ("you now carry the "+item+".");
			}
			//else return ("You carry too many things, you first have to get rid of something before you can carry this item.");
		//}
		else return ("This item is not available here.");	//handle wrong use of method
	}
	
	
	/**
	 * functions as a flac for type of room
	 * @return a String representing the class of a room
	 */
	public String isRoom(){
		return "Room";
	}
    
	//**********************various accessor and mutator methods***********************
	//I did not have enough time to comment them, and they should not be very mysterious
   
	public void setImage(String location)
    {	
    	imageident = location;
    }
    
    public String getImageID()
    {

    	return imageident;
    	
    }
    
	public HashMap getExits() {
		
		return exits;
	}

	
	public boolean changeable(){
		return changable;
	}
	
	
	public Room changeSit(){
		return newSit;
	}

	public HashMap getItems(){
		return items;
	}
	
    public void setItem(String itemdescription, Item item){
    	
    	items.put(itemdescription,item);
    }

	public String getHelp() {
		
		return help;
	}
	
	public void setHelp(String tip){
		help = tip;
	}


	public void removeItem(String itemdesc) {
		items.remove(itemdesc);		
	}

	
	public void setPerson(Person person) {
		roomHasPerson =true;
		this.person = person;		
	}
	
	
	public Person getPerson() {
		
		return person;
	}
	/**
	 * @return Returns the roomHasPerson.
	 */
	public boolean HasPerson() {
		return roomHasPerson;
	}



	/**
	 * @param commandArgument
	 * @return
	 */
	public String buy(String commandArgument) {
		
		return "This action is not available here.";
	}


	/**
	 * @return
	 */
	public boolean photoavailable() {
		
		return photoavailable;
	}
	public void setPhoto(String name) {
		photoid = name;
		photoavailable=true;
		
	}
	public String photo(){
		photoTaken=true;
		return photoid;
	}


	public boolean photoAlreadyTaken() {
		
		return photoTaken;
	}

	public void setPlayer(Player player){
		this.player = player;
	}
	
	public Player getPlayer(){
		return player;
	}
	
	public void timeUpdate(){	 	
	 	player.timeUpdate(1);
	 }


	public void timeUpdate(int i) {
		player.timeUpdate(i);
	}



	/**
	 * @param item
	 */
	public String put(String item) {
		setItem(item,(Item) player.getCarried().get(item));
		player.remove(item);
		return "You layed down your "+ item;
	}
	
}

